@@ -1,6 +1,8 @@
-from zinc_request_processor import ZincRequestProcessor
+from zinc_request_processor import ZincRequestProcessor, ZincError
 from format_price import format_price
+from getpass import getpass
 import sys
+import os
 import json
 
 WELCOME_BANNER = """
@@ -41,10 +43,10 @@ class ValidationHelpers(object):
     @classmethod
     def validate_boolean(klass):
         def validate(x):
-            if (x == "y" or x == "n"):
+            if (x == "y" or x == "n" or x == ""):
                 return True
             else:
-                print "You must enter either 'y' or 'n'"
+                print "You must enter either 'y', 'n', or nothing"
         return validate
 
     @classmethod
@@ -68,6 +70,8 @@ class ValidationHelpers(object):
 
 class ZincWizard(object):
     PROMPTS = {
+        "search_query": "What do you want to buy? (e.g. black cup)",
+        "select_product_name": "Here are your results. Please select a product to purchase.",
         "product_variants": "Please please enter a product URL.",
         "product_quantity": "How many would you like to purchase? (Default: 1)",
         "select_product_variants": "This item comes in multiple variants. Please choose an option.",
@@ -76,9 +80,8 @@ class ZincWizard(object):
         "place_order": "Would you like to place this order? (y)/n",
         "gift": "Do you want this to be shipped as a gift? (y)/n",
         "retailer_credentials": {
-            "start_message": "Would you like to use your Amazon credentials to make the purchase? (y)/n",
-            "username": "What is your Amazon username?",
-            "password": "What is your Amazon password?"
+            "email": "Please enter your Amazon username (usually your email)",
+            "password": "Please enter your Amazon password"
             },
         "shipping_address": {
             "start_message": "\nNow we'd like to get your shipping information.",
@@ -106,44 +109,66 @@ class ZincWizard(object):
             "expiration_year": "Please input your credit card expiration year (e.g. 2017)",
             "security_code": "Please input the CVV security code from your credit card",
             "end_message": "\nYou've finished entering your credit card information!"
-            }
+            },
+        "write_to_zincrc": "Would you like to write the information you just entered to a configuration file (~/.zincrc) so you can make orders more easily in the future? We'll only include your shipping address and a hashed credit card token, so no confidential information will be written to your hard drive."
         }
 
     def __init__(self,
-            product_url = None,
             retailer = "amazon",
             client_token = "public",
-            shipping_address = None,
-            billing_address = None,
-            retailer_credentials = None,
-            credit_card = None,
-            cc_token = None,
+            filename = "~/.zinrc",
             gift = None,
             ):
-        self.response_data = {}
-        self.security_code = None
-        self.product_url = product_url
         self.retailer = retailer
         self.client_token = client_token
-        self.shipping_address = shipping_address
-        self.billing_address = billing_address
-        self.retailer_credentials = retailer_credentials
-        self.credit_card = credit_card
-        self.cc_token = cc_token
+        self.stored_data = self.get_stored_data(filename)
         self.gift = gift
 
+        self.response_data = {}
+        self.product_url = None
+        self.security_code = None
+        self.async_responses = {}
+
+    def get_stored_data(self, filename):
+        if os.path.isfile(filename):
+            with open(filename, 'rb') as f:
+                return json.loads(f.read())
+        return {}
+
+    def retrieve_data(self, key):
+        if key in self.stored_data:
+            return self.stored_data[key]
+        elif key == "shipping_address":
+            print self.PROMPTS[key]["start_message"]
+            return self.get_address(key)
+        elif key == "billing_address":
+            if self.prompt_boolean(self.PROMPTS[key]["start_message"]):
+                return self.response_data["shipping_address"]
+            else:
+                return self.get_address(key)
+
     def start(self):
         print WELCOME_BANNER
-        self.get_product_variants(self.response_data)
-        self.get_shipping_methods(self.response_data)
-        self.get_store_card(self.response_data)
-        self.get_review_order(self.response_data)
-        self.get_place_order(self.response_data)
+        try:
+            self.get_product_name(self.response_data)
+            self.get_product_variants(self.response_data)
+            self.get_retailer_credentials(self.response_data)
+            self.get_shipping_methods(self.response_data)
+            self.get_store_card(self.response_data)
+            self.get_review_order(self.response_data)
+            self.get_place_order(self.response_data)
+        except ZincError as e:
+            print e
+            print "\nRestarting...\n"
+            self.start()
 
-    def prompt(self, prompt, validation=None, max_attempts=3):
+    def prompt(self, prompt, validation=None, max_attempts=3, password=False):
         attempts = 0
         while True:
-            raw = raw_input(prompt + "\n")
+            if password:
+                raw = getpass(prompt + "\n")
+            else:
+                raw = raw_input(prompt + "\n")
             if (validation == None) or (validation != None and validation(raw)):
                 return raw
             else:
@@ -158,10 +183,19 @@ class ZincWizard(object):
             return True
         return False
 
-    def get_product_variants(self, response_data):
-        if self.product_url == None:
-            self.product_url = self.prompt(self.PROMPTS["product_variants"])
+    def get_product_name(self, response_data):
+        search_query = self.prompt(self.PROMPTS["search_query"])
+        print "\nProcessing request...\n"
+        product_name_response = ZincRequestProcessor.process("search_products", {
+                "client_token": self.client_token,
+                "retailer": self.retailer,
+                "search_query": search_query
+            })
 
+        response_data["product_name_response"] = product_name_response
+        self.product_url = self.select_product_name(product_name_response)
+
+    def get_product_variants(self, response_data):
         print "\nProcessing request...\n"
         variants_response = ZincRequestProcessor.process("variant_options", {
                     "client_token": self.client_token,
@@ -171,40 +205,51 @@ class ZincWizard(object):
         response_data["variant_options_response"] = variants_response
         response_data["products"] = self.select_product_variants(variants_response)
 
-    def get_shipping_methods(self, response_data):
-        shipping_address = self.load_file_contents("shipping_address")
+    def get_retailer_credentials(self, response_data):
         print "\nProcessing request...\n"
-        shipping_response = ZincRequestProcessor.process("shipping_methods", {
+        email = self.prompt(self.PROMPTS["retailer_credentials"]["email"])
+        password = self.prompt(self.PROMPTS["retailer_credentials"]["password"], password=True)
+        response_data["retailer_credentials"] = {
+                "email": email,
+                "password": password
+                }
+
+    def get_shipping_methods(self, response_data):
+        self.shipping_address = self.retrieve_data("shipping_address")
+        print "\nProcessing request...\n"
+        shipping_response = ZincRequestProcessor.process_async("shipping_methods", {
                     "client_token": self.client_token,
                     "retailer": self.retailer,
                     "products": response_data["products"],
-                    "shipping_address": shipping_address
+                    "shipping_address": self.shipping_address,
+                    "retailer_credentials": response_data["retailer_credentials"]
                 })
 
-        response_data["shipping_response"] = shipping_response
-        response_data["shipping_address"] = shipping_address
-        response_data["shipping_method_id"] = self.select_shipping_methods(shipping_response)
+        self.async_responses["shipping_response"] = shipping_response
 
     def get_store_card(self, response_data):
-        if self.cc_token == None:
-            cc_data = self.load_file_contents("credit_card")
-            billing_address = self.load_file_contents("billing_address")
+        cc_token = self.retrieve_data("cc_token")
+        if cc_token == None:
+            cc_data = self.get_credit_card_information()
+            self.billing_address = self.retrieve_data("billing_address")
             print "\nProcessing request...\n"
             store_card_response = ZincRequestProcessor.process("store_card", {
                         "client_token": self.client_token,
                         "retailer": self.retailer,
-                        "billing_address": billing_address,
+                        "billing_address": self.billing_address,
                         "number": cc_data["number"],
                         "expiration_month": cc_data["expiration_month"],
                         "expiration_year": cc_data["expiration_year"]
                     })
 
             response_data["store_card_response"] = store_card_response
-            response_data["cc_token"] = store_card_response["cc_token"]
+            self.stored_data["cc_token"] = store_card_response["cc_token"]
 
     def get_review_order(self, response_data):
+        shipping_method_id = self.select_shipping_methods(
+                self.async_responses["shipping_response"].get_response())
         payment_method = {
-                "cc_token": response_data["cc_token"],
+                "cc_token": self.retrieve_data("cc_token"),
                 "security_code": self.get_security_code()
                 }
         is_gift = self.get_is_gift()
@@ -213,16 +258,18 @@ class ZincWizard(object):
                     "client_token": self.client_token,
                     "retailer": self.retailer,
                     "products": response_data["products"],
-                    "shipping_address": response_data["shipping_address"],
+                    "shipping_address": self.shipping_address,
                     "is_gift": is_gift,
-                    "shipping_method_id": response_data["shipping_method_id"],
+                    "shipping_method_id": shipping_method_id,
                     "payment_method": payment_method,
-                    "customer_email": "support@zinc.io"
+                    "customer_email": "support@zinc.io",
+                    "retailer_credentials": response_data["retailer_credentials"]
                 })
 
         response_data["review_order_response"] = review_order_response
 
     def get_place_order(self, response_data):
+        self.write_to_zincrc()
         self.print_price_components(response_data)
         if self.prompt_boolean(self.PROMPTS["place_order"]):
             print "\nProcessing request...\n"
@@ -248,6 +295,15 @@ class ZincWizard(object):
             self.print_indent("Promotion:        %s" % format_price(components["promotion"]))
         self.print_indent("Total:            %s" % format_price(components["total"]))
 
+    def write_to_zincrc(self):
+        if self.prompt_boolean(self.PROMPTS["write_to_zincrc"]):
+            data = {
+                "shipping_address": self.shipping_address,
+                "cc_token": self.cc_token
+                }
+            with open("~/.zincrc", 'wb') as f:
+                f.write(json.dumps(data))
+
     def print_indent(self, value):
         print "    ", value
 
@@ -261,21 +317,6 @@ class ZincWizard(object):
             return self.security_code
         return self.prompt(self.PROMPTS["security_code"])
 
-    def load_file_contents(self, filetype):
-        if getattr(self, filetype) != None:
-            with open(getattr(self, filetype), 'rb') as f:
-                return json.loads(f.read())
-        elif filetype == "shipping_address":
-            print self.PROMPTS[filetype]["start_message"]
-            return self.get_address(filetype)
-        elif filetype == "billing_address":
-            if self.prompt_boolean(self.PROMPTS[filetype]["start_message"]):
-                return self.response_data["shipping_address"]
-            else:
-                return self.get_address(filetype)
-        elif filetype == "credit_card":
-            return self.get_credit_card_information()
-
     def get_credit_card_information(self):
         print self.PROMPTS["credit_card"]["start_message"]
         response = {}
@@ -314,6 +355,26 @@ class ZincWizard(object):
         prompt += "\n".join(description_list)
         return prompt
 
+    def get_quantity(self):
+        quantity = self.prompt(self.PROMPTS["product_quantity"]).strip()
+        if quantity == "":
+            return 1
+        else:
+            return quantity
+
+    def select_product_name(self, response):
+        descriptions = []
+        collector = []
+        for i in xrange(len(response["results"])):
+            current = response["results"][i]
+            descriptions.append(str(i) + ") " + current["title"])
+            collector.append(current["product_url"])
+
+        prompt = self.build_prompt(self.PROMPTS["select_product_name"], descriptions)
+        collected_number = self.prompt(prompt,
+                ValidationHelpers.validate_number(len(descriptions)-1))
+        return collector[int(collected_number)]
+
     def select_product_variants(self, variants_response):
         descriptions = []
         product_ids = []
@@ -342,13 +403,6 @@ class ZincWizard(object):
                 "quantity": quantity
                 }]
 
-    def get_quantity(self):
-        quantity = self.prompt(self.PROMPTS["product_quantity"]).strip()
-        if quantity == "":
-            return 1
-        else:
-            return quantity
-
     def select_shipping_methods(self, shipping_response):
         descriptions = []
         shipping_ids = []
